/*@@
  @file      StandardVariables.F
  @desc 
     Calculate the Standard ADM variables from the BS ones
  @enddesc 
@@*/

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"
#include "cctk_Functions.h"

/*@@
  @routine    ADM_BSSN_StandardVariables
  @desc 
     Calculate the Standard ADM variables from the BS ones

  @enddesc 
  @calls     
  @calledby 
  @history
  @hauthor 
  @hdate  @hdesc 
  @endhistory 
@@*/

      subroutine ADM_BSSN_StandardVariables(CCTK_ARGUMENTS)

      USE ADM_BSSN_Scalars

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS
      DECLARE_CCTK_FUNCTIONS

      integer :: i,j,k
      integer :: nx,ny,nz

      CCTK_REAL :: aux,fac,third

c     Numbers.

      third = 1.0D0/3.0D0

c     Grid.

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

c     Find ADM variables.

      if (conformal_state == 0) then

         do k=1,nz
            do j=1,ny
               do i=1,nx

                  fac = exp(4.0D0*ADM_BS_phi(i,j,k))

                  gxx(i,j,k) = fac*ADM_BS_gxx(i,j,k)
                  gxy(i,j,k) = fac*ADM_BS_gxy(i,j,k)
                  gxz(i,j,k) = fac*ADM_BS_gxz(i,j,k)
                  gyy(i,j,k) = fac*ADM_BS_gyy(i,j,k)
                  gyz(i,j,k) = fac*ADM_BS_gyz(i,j,k)
                  gzz(i,j,k) = fac*ADM_BS_gzz(i,j,k)

                  aux = third*ADM_BS_K(i,j,k)

                  kxx(i,j,k) = fac*(ADM_BS_Axx(i,j,k) + aux*ADM_BS_gxx(i,j,k))
                  kxy(i,j,k) = fac*(ADM_BS_Axy(i,j,k) + aux*ADM_BS_gxy(i,j,k))
                  kxz(i,j,k) = fac*(ADM_BS_Axz(i,j,k) + aux*ADM_BS_gxz(i,j,k))
                  kyy(i,j,k) = fac*(ADM_BS_Ayy(i,j,k) + aux*ADM_BS_gyy(i,j,k))
                  kyz(i,j,k) = fac*(ADM_BS_Ayz(i,j,k) + aux*ADM_BS_gyz(i,j,k))
                  kzz(i,j,k) = fac*(ADM_BS_Azz(i,j,k) + aux*ADM_BS_gzz(i,j,k))

               end do
            end do
         end do

      else

         do k=1,nz
            do j=1,ny
               do i=1,nx

                  fac = exp(4.0d0*ADM_BS_phi(i,j,k))

                  gxx(i,j,k) = fac*ADM_BS_gxx(i,j,k)
                  gxy(i,j,k) = fac*ADM_BS_gxy(i,j,k)
                  gxz(i,j,k) = fac*ADM_BS_gxz(i,j,k)
                  gyy(i,j,k) = fac*ADM_BS_gyy(i,j,k)
                  gyz(i,j,k) = fac*ADM_BS_gyz(i,j,k)
                  gzz(i,j,k) = fac*ADM_BS_gzz(i,j,k)

                  fac = fac*psi(i,j,k)**4
                  aux = third*ADM_BS_K(i,j,k)

                  kxx(i,j,k) = fac*(ADM_BS_Axx(i,j,k) + aux*ADM_BS_gxx(i,j,k))
                  kxy(i,j,k) = fac*(ADM_BS_Axy(i,j,k) + aux*ADM_BS_gxy(i,j,k))
                  kxz(i,j,k) = fac*(ADM_BS_Axz(i,j,k) + aux*ADM_BS_gxz(i,j,k))
                  kyy(i,j,k) = fac*(ADM_BS_Ayy(i,j,k) + aux*ADM_BS_gyy(i,j,k))
                  kyz(i,j,k) = fac*(ADM_BS_Ayz(i,j,k) + aux*ADM_BS_gyz(i,j,k))
                  kzz(i,j,k) = fac*(ADM_BS_Azz(i,j,k) + aux*ADM_BS_gzz(i,j,k))

               end do
            end do
         end do

      end if

      end subroutine ADM_BSSN_StandardVariables

      subroutine ADM_BSSN_LapseChange(CCTK_ARGUMENTS)

      USE ADM_BSSN_Scalars

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS
      DECLARE_CCTK_FUNCTIONS

c     Find the change in the lapse.
      if (evolveLapse.and.
     .   (CCTK_EQUALS(lapsesource,"standard"))) then
         ADM_BS_dtalp = (alp - alp_p) / CCTK_DELTA_TIME
      end if

      end subroutine ADM_BSSN_LapseChange
