 /*@@
   @file      Whisky_Boundaries.F90
   @date      Sat Jan 26 01:01:14 2002
   @author    
   @desc 
   The two routines for dealing with boundary conditions.
   @enddesc 
 @@*/

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"
#include "cctk_Functions.h"

#include "util_Table.h"

 /*@@
   @routine    Whisky_InitSymBound
   @date       Sat Jan 26 01:03:04 2002
   @author     Ian Hawke
   @desc 
   Sets up the symmetries at the boundaries of the hydrodynamical variables.
   @enddesc 
   @calls     
   @calledby   
   @history 
   Direct translation of routines from GR3D, GRAstro_Hydro, 
   written by Mark Miller, or WaveToy routines, or...
   @endhistory 

@@*/

subroutine Whisky_InitSymBound(CCTK_ARGUMENTS)

  implicit none
  
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  integer, dimension(3) :: sym
  integer :: ierr
  
  sym(1) = 1
  sym(2) = 1
  sym(3) = 1
  
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::rho")
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::press")
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::dens")
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::tau")
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::w_lorentz")
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::eps")

  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::tracer")
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::cons_tracer")
  
  sym(1) = -1
  sym(2) = 1
  sym(3) = 1
  
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::velx")
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::sx")
  
  sym(1) = 1
  sym(2) = -1
  sym(3) = 1
  
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::vely")
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::sy")
  
  sym(1) = 1
  sym(2) = 1
  sym(3) = -1
  
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::velz")
  call SetCartSymVN(ierr, cctkGH, sym, "Whisky::sz")
  
end subroutine Whisky_InitSymBound

 /*@@
   @routine    Whisky_Boundaries
   @date       Sat Jan 26 01:04:04 2002
   @author     
   @desc 
   Calls the appropriate boundary routines
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

subroutine Whisky_Boundaries(CCTK_ARGUMENTS)
  
  implicit none
  
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS
  DECLARE_CCTK_FUNCTIONS
  
  integer, dimension(3) :: sw
  integer :: ierr
  CCTK_REAL :: pi = 3.141569d0, zero = 0.d0
  integer :: i,j,k

  CCTK_INT, parameter :: faces=CCTK_ALL_FACES
  CCTK_INT, parameter :: ione=1
  
  sw = Whisky_stencil

!!$Symmetry boundaries first  

!!$  call CartSymGN(ierr,cctkGH,"Whisky::Whisky_cons_var")
!!$  call CartSymGN(ierr,cctkGH,"Whisky::Whisky_prim_var")

!!$Flat boundaries if required  

  if (CCTK_EQUALS(bound,"flat")) then
    ierr = Boundary_SelectGroupForBC(cctkGH, faces, whisky_stencil, -ione, &
         "Whisky::Whisky_cons_scalar_var", "Flat")
    ierr = Boundary_SelectGroupForBC(cctkGH, faces, whisky_stencil, -ione, &
         "Whisky::Whisky_prim_scalar_var", "Flat")
    ierr = Boundary_SelectGroupForBC(cctkGH, faces, whisky_stencil, -ione, &
         "Whisky::Whisky_cons_vector_var", "Flat")
    ierr = Boundary_SelectGroupForBC(cctkGH, faces, whisky_stencil, -ione, &
         "Whisky::Whisky_prim_vector_var", "Flat")
!!$    call BndFlatGN(ierr,cctkGH,sw,"Whisky::Whisky_cons_var")
!!$    call BndFlatGN(ierr,cctkGH,sw,"Whisky::Whisky_prim_var")
  endif

  if (CCTK_EQUALS(bound,"none")) then
    ierr = Boundary_SelectGroupForBC(cctkGH, faces, whisky_stencil, -ione, &
         "Whisky::Whisky_cons_scalar_var", "None")
    ierr = Boundary_SelectGroupForBC(cctkGH, faces, whisky_stencil, -ione, &
         "Whisky::Whisky_prim_scalar_var", "None")
    ierr = Boundary_SelectGroupForBC(cctkGH, faces, whisky_stencil, -ione, &
         "Whisky::Whisky_cons_vector_var", "None")
    ierr = Boundary_SelectGroupForBC(cctkGH, faces, whisky_stencil, -ione, &
         "Whisky::Whisky_prim_vector_var", "None")
!!$    call  BndStaticGN(ierr,cctkGH,sw,"Whisky::Whisky_cons_var")
!!$    call  BndStaticGN(ierr,cctkGH,sw,"Whisky::Whisky_prim_var")
  end if

  if (CCTK_EQUALS(bound,"static")) then
    ierr = Boundary_SelectGroupForBC(cctkGH, faces, whisky_stencil, -ione, &
         "Whisky::Whisky_cons_scalar_var", "Static")
    ierr = Boundary_SelectGroupForBC(cctkGH, faces, whisky_stencil, -ione, &
         "Whisky::Whisky_prim_scalar_var", "Static")
    ierr = Boundary_SelectGroupForBC(cctkGH, faces, whisky_stencil, -ione, &
         "Whisky::Whisky_cons_vector_var", "Static")
    ierr = Boundary_SelectGroupForBC(cctkGH, faces, whisky_stencil, -ione, &
         "Whisky::Whisky_prim_vector_var", "Static")
!!$    call  BndStaticGN(ierr,cctkGH,sw,"Whisky::Whisky_cons_var")
!!$    call  BndStaticGN(ierr,cctkGH,sw,"Whisky::Whisky_prim_var")
  end if

  if (CCTK_EQUALS(bound,"scalar")) then
    call CCTK_WARN(0, "Until somebody uses this I see no reason to support it")
!!$    call  BndScalarGN(ierr,cctkGH,sw,zero,"Whisky::Whisky_cons_var")
!!$    call  BndScalarGN(ierr,cctkGH,sw,zero,"Whisky::Whisky_prim_var")
  end if

end subroutine Whisky_Boundaries





