 /*@@
   @file      Whisky_Reconstruct.F90
   @date      Sat Jan 26 02:13:25 2002
   @author    
   @desc 
   Wrapper routine to perform the reconstruction.
   @enddesc 
 @@*/

#include "cctk.h"
#include "cctk_Arguments.h"
#include "cctk_Functions.h"
#include "cctk_Parameters.h"

 /*@@
   @routine    Reconstruction
   @date       Sat Jan 26 02:13:47 2002
   @author     Luca Baiotti, Ian Hawke
   @desc 
   A wrapper routine to do reconstruction. Currently just does
   TVD on the primitive variables.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

subroutine Reconstruction(CCTK_ARGUMENTS)
  
  implicit none
  
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS
  DECLARE_CCTK_FUNCTIONS

  integer :: nx, ny, nz, i, j, k
  
  nx = cctk_lsh(1)
  ny = cctk_lsh(2)
  nz = cctk_lsh(3)

!!$  Currently only option is reconstruction on primitive variables.
!!$  Should change this.


  if (CCTK_EQUALS(recon_method,"tvd")) then
    if (CCTK_EQUALS(recon_vars,"primitive")) then
      call tvdreconstruct(nx, ny, nz, xoffset, yoffset, zoffset, &
           rho, rhoplus, rhominus)
      call tvdreconstruct(nx, ny, nz, xoffset, yoffset, zoffset, &
           velx, velxplus, velxminus)
      call tvdreconstruct(nx, ny, nz, xoffset, yoffset, zoffset, &
           vely, velyplus, velyminus)
      call tvdreconstruct(nx, ny, nz, xoffset, yoffset, zoffset, &
           velz, velzplus, velzminus)
      call tvdreconstruct(nx, ny, nz, xoffset, yoffset, zoffset, &
           eps, epsplus, epsminus)
    else if (CCTK_EQUALS(recon_vars,"conservative")) then
      call tvdreconstruct(nx, ny, nz, xoffset, yoffset, zoffset, &
           dens, densplus, densminus)
      call tvdreconstruct(nx, ny, nz, xoffset, yoffset, zoffset, &
           sx, sxplus, sxminus)
      call tvdreconstruct(nx, ny, nz, xoffset, yoffset, zoffset, &
           sy, syplus, syminus)
      call tvdreconstruct(nx, ny, nz, xoffset, yoffset, zoffset, &
           sz, szplus, szminus)
      call tvdreconstruct(nx, ny, nz, xoffset, yoffset, zoffset, &
           tau, tauplus, tauminus)
    else
      call CCTK_WARN(0, "Variable type to reconstruct not recognized.")
    end if
  else if (CCTK_EQUALS(recon_method,"ppm")) then
    if (flux_direction == 1) then
      do k = whisky_stencil, cctk_lsh(3) - whisky_stencil + 1
        do j = whisky_stencil, cctk_lsh(2) - whisky_stencil + 1
          call SimplePPM_1d(whisky_eos_handle,cctk_lsh(1),CCTK_DELTA_SPACE(1),&
               rho(:,j,k),velx(:,j,k),vely(:,j,k),velz(:,j,k),eps(:,j,k),&
               rhominus(:,j,k),velxminus(:,j,k),velyminus(:,j,k),&
               velzminus(:,j,k),epsminus(:,j,k),rhoplus(:,j,k),&
               velxplus(:,j,k),velyplus(:,j,k),velzplus(:,j,k),epsplus(:,j,k))
        end do
      end do
    else if (flux_direction == 2) then
      do k = whisky_stencil, cctk_lsh(3) - whisky_stencil + 1
        do j = whisky_stencil, cctk_lsh(1) - whisky_stencil + 1
          call SimplePPM_1d(whisky_eos_handle,cctk_lsh(2),CCTK_DELTA_SPACE(2),&
               rho(j,:,k),vely(j,:,k),velz(j,:,k),velx(j,:,k),eps(j,:,k),&
               rhominus(j,:,k),velyminus(j,:,k),velzminus(j,:,k),&
               velxminus(j,:,k),epsminus(j,:,k),rhoplus(j,:,k),&
               velyplus(j,:,k),velzplus(j,:,k),velxplus(j,:,k),epsplus(j,:,k))
        end do
      end do
    else if (flux_direction == 3) then
      do k = whisky_stencil, cctk_lsh(2) - whisky_stencil + 1
        do j = whisky_stencil, cctk_lsh(1) - whisky_stencil + 1
          call SimplePPM_1d(whisky_eos_handle,cctk_lsh(3),CCTK_DELTA_SPACE(3),&
               rho(j,k,:),velz(j,k,:),velx(j,k,:),vely(j,k,:),eps(j,k,:),&
               rhominus(j,k,:),velzminus(j,k,:),velxminus(j,k,:),&
               velyminus(j,k,:),epsminus(j,k,:),rhoplus(j,k,:),&
               velzplus(j,k,:),velxplus(j,k,:),velyplus(j,k,:),epsplus(j,k,:))
        end do
      end do
    else
      call CCTK_WARN(0, "Flux direction not x,y,z")
    end if
  else if (CCTK_EQUALS(recon_method,"eno")) then
    if (flux_direction == 1) then
      do k = whisky_stencil, cctk_lsh(3) - whisky_stencil + 1
        do j = whisky_stencil, cctk_lsh(2) - whisky_stencil + 1
          if (CCTK_EQUALS(recon_vars,"primitive")) then
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(1),&
                 rho(:,j,k),rhominus(:,j,k),rhoplus(:,j,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(1),&
                 velx(:,j,k),velxminus(:,j,k),velxplus(:,j,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(1),&
                 vely(:,j,k),velyminus(:,j,k),velyplus(:,j,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(1),&
                 velz(:,j,k),velzminus(:,j,k),velzplus(:,j,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(1),&
                 eps(:,j,k),epsminus(:,j,k),epsplus(:,j,k))
          else if (CCTK_EQUALS(recon_vars,"conservative")) then
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(1),&
                 dens(:,j,k),densminus(:,j,k),densplus(:,j,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(1),&
                 sx(:,j,k),sxminus(:,j,k),sxplus(:,j,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(1),&
                 sy(:,j,k),syminus(:,j,k),syplus(:,j,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(1),&
                 sz(:,j,k),szminus(:,j,k),szplus(:,j,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(1),&
                 tau(:,j,k),tauminus(:,j,k),tauplus(:,j,k))
          else
            call CCTK_WARN(0, "Variable type to reconstruct not recognized.")
          end if
        end do
      end do
    else if (flux_direction == 2) then
      do k = whisky_stencil, cctk_lsh(3) - whisky_stencil + 1
        do j = whisky_stencil, cctk_lsh(1) - whisky_stencil + 1
          if (CCTK_EQUALS(recon_vars,"primitive")) then
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(2),&
                 rho(j,:,k),rhominus(j,:,k),rhoplus(j,:,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(2),&
                 velx(j,:,k),velxminus(j,:,k),velxplus(j,:,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(2),&
                 vely(j,:,k),velyminus(j,:,k),velyplus(j,:,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(2),&
                 velz(j,:,k),velzminus(j,:,k),velzplus(j,:,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(2),&
                 eps(j,:,k),epsminus(j,:,k),epsplus(j,:,k))
          else if (CCTK_EQUALS(recon_vars,"conservative")) then
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(2),&
                 dens(j,:,k),densminus(j,:,k),densplus(j,:,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(2),&
                 sx(j,:,k),sxminus(j,:,k),sxplus(j,:,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(2),&
                 sy(j,:,k),syminus(j,:,k),syplus(j,:,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(2),&
                 sz(j,:,k),szminus(j,:,k),szplus(j,:,k))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(2),&
                 tau(j,:,k),tauminus(j,:,k),tauplus(j,:,k))
          else
            call CCTK_WARN(0, "Variable type to reconstruct not recognized.")
          end if
        end do
      end do
    else if (flux_direction == 3) then
      do k = whisky_stencil, cctk_lsh(2) - whisky_stencil + 1
        do j = whisky_stencil, cctk_lsh(1) - whisky_stencil + 1
          if (CCTK_EQUALS(recon_vars,"primitive")) then
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(3),&
                 rho(j,k,:),rhominus(j,k,:),rhoplus(j,k,:))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(3),&
                 velx(j,k,:),velxminus(j,k,:),velxplus(j,k,:))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(3),&
                 vely(j,k,:),velyminus(j,k,:),velyplus(j,k,:))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(3),&
                 velz(j,k,:),velzminus(j,k,:),velzplus(j,k,:))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(3),&
                 eps(j,k,:),epsminus(j,k,:),epsplus(j,k,:))
          else if (CCTK_EQUALS(recon_vars,"conservative")) then
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(3),&
                 dens(j,k,:),densminus(j,k,:),densplus(j,k,:))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(3),&
                 sx(j,k,:),sxminus(j,k,:),sxplus(j,k,:))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(3),&
                 sy(j,k,:),syminus(j,k,:),syplus(j,k,:))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(3),&
                 sz(j,k,:),szminus(j,k,:),szplus(j,k,:))
            call Whisky_ENOReconstruct1d(eno_order,cctk_lsh(3),&
                 tau(j,k,:),tauminus(j,k,:),tauplus(j,k,:))
          else
            call CCTK_WARN(0, "Variable type to reconstruct not recognized.")
          end if
        end do
      end do
    else
      call CCTK_WARN(0, "Flux direction not x,y,z")
    end if
  else
    call CCTK_WARN(0, "Reconstruction method not recognized!")
  end if

  if (CCTK_EQUALS(recon_vars,"primitive").or.&
       CCTK_EQUALS(recon_method,"ppm")) then
    call primitive2conservative(CCTK_PASS_FTOF)
  else if (CCTK_EQUALS(recon_vars,"conservative")) then
    call Conservative2PrimitiveBounds(CCTK_PASS_FTOF)
  else
    call CCTK_WARN(0,"Variable type to reconstruct not recognized.")
  end if

  return

end subroutine Reconstruction

